package NANDDump;
import gnu.io.CommPortIdentifier;
import java.awt.Color;
import java.awt.Toolkit;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.BufferedWriter;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashSet;
import java.util.Iterator;
import javax.swing.JFileChooser;
import javax.swing.SwingWorker;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyleContext;
import javax.swing.text.StyledDocument;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import java.util.HashSet;

// RXTX lib required, see http://rxtx.qbang.org/wiki/index.php/Main_Page
import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;

public class MainFrame extends javax.swing.JFrame implements PropertyChangeListener
{

    class NandConfig
    {

        public String name;
        public int sizeID;
        public int sizePage;
        public int sizePageOOB;
        public int sizeBlock;
        public int numBlocks;

        public NandConfig(String _name, int _sizeID, int _sizePage, int _sizePageOOB, int _sizeBlock, int _numBlocks)
        {
            name = _name;
            sizeID = _sizeID;
            sizePage = _sizePage;
            sizePageOOB = _sizePageOOB;
            sizeBlock = _sizeBlock;
            numBlocks = _numBlocks;
        }

        @Override
        public String toString()
        {
            long lSize = ((long) sizeBlock * (long) numBlocks) / (1024 * 1024);
            return name + " " + lSize + "MiB";
        }

        public int getRawPageSize()
        {
            return sizePage + sizePageOOB;
        }

        public int getRawBlockSize()
        {
            return sizeBlock + ((sizeBlock / sizePage) * sizePageOOB);
        }

        public long getRawChipSize()
        {
            return getRawBlockSize() * numBlocks;
        }
    }

    SerialPort serialPort;
    InputStream serialIn;
    OutputStream serialOut;
    NandConfig selectedNandConfig;
    private DumpTask dumpTask;
    private StyledDocument log;

    class DumpTask extends SwingWorker<Void, Void>
    {
        boolean aborted = false;
        String fileName;
        int blockFrom;
        int blockTo;
        long bytesRead = 0;
        long timeElapsed = 0;
        int readProgress = 0;
        int counter = 0;

        public DumpTask(String _fileName, int _blockFrom, int _blockTo)
        {
            fileName = _fileName;
            blockFrom = _blockFrom;
            blockTo = _blockTo;
        }

        @Override
        public Void doInBackground()
        {
            try
            {
                writeNandConfig();
                resetChip();
                readChipID();
                BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));

                FileOutputStream fileOut = null;
                fileOut = new FileOutputStream(fileName);

                byte[] data = new byte[selectedNandConfig.getRawBlockSize()];
                long totalBytes = (long)data.length * ((long)blockTo - (long)blockFrom + 1);
                long totalBytesRead = 0;

                logln("Dumping blocks " + blockFrom + " to " + blockTo +
                      " (" + totalBytes + " bytes total) to file \"" +
                      fileName + "\"");
                long timeStart = System.currentTimeMillis();
                long timeLastProgress = timeStart;
                for(int block = blockFrom; block <= blockTo; block++)
                {
                    if(aborted)
                    {
                        break;
                    }
                    bw.write("5," + block + "\n");
                    bw.flush();
                    int bytesToRead = data.length;
                    while(bytesToRead > 0)
                    {
                        int nRead = serialIn.read(data, data.length - bytesToRead, bytesToRead);
                        if(nRead > 0)
                        {
                            bytesToRead -= nRead;
                            totalBytesRead += nRead;
                        }
                        if((System.currentTimeMillis() - timeLastProgress) > 250)
                        {
                            setProgress((int)((totalBytesRead * 100) / totalBytes));
                            timeLastProgress = System.currentTimeMillis();
                        }
                    }
                    String result = serialReadLine();
                    logln("Read block " + block + " result: " + result);
                    bytesRead += data.length - bytesToRead;
                    fileOut.write(data);

                }
                fileOut.close();
                timeElapsed = System.currentTimeMillis() - timeStart;
                setProgress(100);
            }
            catch(Exception e)
            {
                logln("Error: " + e.getMessage());
            }

            return null;
        }

        @Override
        public void done()
        {
            if(aborted)
            {
                logln("Dump canceled.");
            }
            else
            {
                Toolkit.getDefaultToolkit().beep();
                double secElapsed = ((double) timeElapsed) / 1000;
                double speed = ((double) bytesRead / (double) 0x100000) / secElapsed;
                logln(bytesRead + " bytes dumped in " + String.format("%.3f", secElapsed) +
                      " seconds (" + String.format("%.3f", speed) + " MiB/s).");
            }
            dumpTask = null;
            updateControls();
        }

        public void abort()
        {
            aborted = true;
        }
    }

    public void propertyChange(PropertyChangeEvent evt)
    {
        if(evt.getPropertyName() == "progress")
        {
            int progress = (Integer) evt.getNewValue();
            progressBar.setValue(progress);
            progressBar.setString(progress + "%");
        }
    }

    public MainFrame()
    {

        initComponents();

        ((AbstractDocument) textFieldBlockFrom.getDocument()).setDocumentFilter(new NumericFilter());
        ((AbstractDocument) textFieldBlockTo.getDocument()).setDocumentFilter(new NumericFilter());

        chipSelect.addItem(new NandConfig("K9GAG08U0E", 6, 8192, 436, 1024 * 1024, 2076));
        chipSelect.addItem(new NandConfig("K9F2G08U0M", 4, 2048, 64, 128 * 1024, 2048));

        StyleContext sc = StyleContext.getDefaultStyleContext();
        AttributeSet as = sc.addAttribute(SimpleAttributeSet.EMPTY, StyleConstants.Foreground, Color.BLACK);
        as = sc.addAttribute(as, StyleConstants.FontFamily, "Lucida Console");
        as = sc.addAttribute(as, StyleConstants.Alignment, StyleConstants.ALIGN_JUSTIFIED);

        textPane.setCaretPosition(textPane.getDocument().getLength());
        textPane.setCharacterAttributes(as, false);

        textFieldFile.setText(System.getProperty("user.dir") + System.getProperty("file.separator") + "nanddump.bin");

        updateSerialSelect();
        updateControls();
    }

    void logln(String str)
    {
        try
        {
            log.insertString(log.getLength(), str + "\n", null);
            textPane.setCaretPosition(textPane.getDocument().getLength());
        }
        catch(BadLocationException e)
        {
        }
    }

    void writeNandConfig() throws IOException
    {
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));
        bw.write("2," + selectedNandConfig.sizeID + "," + selectedNandConfig.sizePage + "," + selectedNandConfig.sizePageOOB + ","
                + selectedNandConfig.sizeBlock + "," + selectedNandConfig.numBlocks + "\n");
        bw.flush();
        logln("Write config result: " + serialReadLine());
    }

    void readNandConfig() throws IOException
    {
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));
        bw.write("1\n");
        bw.flush();
        logln("Read config result: " + serialReadLine());
    }

    void readDebugInfo() throws IOException
    {
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));
        bw.write("666\n");
        bw.flush();
        logln("Read debug info result: " + serialReadLine());
    }

    void readChipID() throws IOException
    {
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));
        bw.write("3\n");
        bw.flush();
        logln("Read chip ID result: " + serialReadLine());
    }

    void resetChip() throws IOException
    {
        BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(serialOut));
        bw.write("6\n");
        bw.flush();
        logln("Reset chip result: " + serialReadLine());
    }
    
    String serialReadLine() throws IOException
    {
        String str = new String();
        char c;
        do
        {
            c = (char) (serialIn.read() & 0xFF);
            str += c;
        }
        while(c != '\n');
        return str.trim();
    }

    HashSet<CommPortIdentifier> serialGetPortIdentifiers()
    {
        HashSet<CommPortIdentifier> setResult = new HashSet<CommPortIdentifier>();
        Enumeration enumPortIdentifiers = CommPortIdentifier.getPortIdentifiers();
        while(enumPortIdentifiers.hasMoreElements())
        {
            CommPortIdentifier portIdentifier = (CommPortIdentifier)enumPortIdentifiers.nextElement();
            if(portIdentifier.getPortType() == CommPortIdentifier.PORT_SERIAL)
            {
                setResult.add(portIdentifier);
            }
        }
        return setResult;
    }

    void serialConnect()
    {
        String port = (String)serialSelect.getSelectedItem();
        try
        {
            CommPortIdentifier portIdentifier = CommPortIdentifier.getPortIdentifier(port);
            if(portIdentifier.isCurrentlyOwned())
            {
                logln("Error: Serial port " + portIdentifier + " is already in use.");
            }
            else
            {
                serialPort = (SerialPort)portIdentifier.open("NANDDump", 2000);
                serialPort.setSerialPortParams(115200,
                                               SerialPort.DATABITS_8,
                                               SerialPort.STOPBITS_1,
                                               SerialPort.PARITY_NONE);
                serialIn = serialPort.getInputStream();
                serialOut = serialPort.getOutputStream();
            }
            logln("Connected to serial port " + portIdentifier.getName() + ".");
        }
        catch(Exception e)
        {
            logln("Error: Failed to connect to serial port " + port + ": " + e);
            serialPort = null;
        }
    }

    void serialDisconnect()
    {
        if(serialIn != null)
        {
            try {serialIn.close();} catch(IOException e) {}
            serialIn = null;
        }
        if(serialOut != null)
        {
            try {serialOut.close();} catch(IOException e) {}
            serialOut = null;
        }

        if(serialPort != null)
        {
            logln("Disconnected from serial port.");
            serialPort.close();
            serialPort = null;
        }
    }

    void updateSerialSelect()
    {
        serialSelect.removeAllItems();

        HashSet setIdentifiers = serialGetPortIdentifiers();
        Iterator<CommPortIdentifier> itIdentifiers = setIdentifiers.iterator();
        while(itIdentifiers.hasNext())
        {
            serialSelect.addItem(itIdentifiers.next().getName());
        }
    }

    void updateControls()
    {
        if(serialPort != null)
        {
            connectButton.setText("Disconnect");
            dumpButton.setEnabled(true);
            if(dumpTask != null)
            {
                dumpButton.setText("Abort");
                connectButton.setEnabled(false);
            }
            else
            {
                dumpButton.setText("Dump");
                connectButton.setEnabled(true);
                progressBar.setString("0%");
            }
        }
        else
        {
            connectButton.setText("Connect");
            dumpButton.setEnabled(false);
            dumpButton.setText("Dump");
            progressBar.setString("0%");
        }
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents()
    {

        saveButton1 = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        progressBar = new javax.swing.JProgressBar();
        serialSelect = new javax.swing.JComboBox();
        dumpButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        chipSelect = new javax.swing.JComboBox();
        jLabel3 = new javax.swing.JLabel();
        textFieldBlockFrom = new javax.swing.JTextField();
        jLabel4 = new javax.swing.JLabel();
        textFieldBlockTo = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        textFieldFile = new javax.swing.JTextField();
        browseButton = new javax.swing.JButton();
        connectButton = new javax.swing.JButton();
        jScrollPane2 = new javax.swing.JScrollPane();
        textPane = new javax.swing.JTextPane();

        saveButton1.setText("Save Dump");
        saveButton1.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                saveButton1ActionPerformed(evt);
            }
        });

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("NANDDump");
        setBounds(new java.awt.Rectangle(0, 0, 0, 0));
        setMinimumSize(new java.awt.Dimension(800, 600));

        progressBar.setStringPainted(true);

        serialSelect.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                serialSelectActionPerformed(evt);
            }
        });

        dumpButton.setText("Dump");
        dumpButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                dumpButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Serial Port:");

        jLabel2.setText("Nand Chip:");

        chipSelect.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                chipSelectActionPerformed(evt);
            }
        });

        jLabel3.setText("From Block:");
        jLabel3.setToolTipText("");

        textFieldBlockFrom.setName(""); // NOI18N
        textFieldBlockFrom.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                textFieldBlockFromFocusLost(evt);
            }
        });

        jLabel4.setText("To Block:");
        jLabel4.setToolTipText("");

        textFieldBlockTo.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                textFieldBlockToFocusLost(evt);
            }
        });

        jLabel5.setText("Output File:");
        jLabel5.setToolTipText("");

        textFieldFile.setEditable(false);
        textFieldFile.setName(""); // NOI18N
        textFieldFile.addFocusListener(new java.awt.event.FocusAdapter()
        {
            public void focusLost(java.awt.event.FocusEvent evt)
            {
                textFieldFileFocusLost(evt);
            }
        });

        browseButton.setText("...");
        browseButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                browseButtonActionPerformed(evt);
            }
        });

        connectButton.setText("Connect");
        connectButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                connectButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .add(25, 25, 25)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel5)
                            .add(jLabel2, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .add(443, 443, 443))
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(dumpButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(progressBar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                            .add(jPanel1Layout.createSequentialGroup()
                                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jLabel3)
                                    .add(jLabel4)
                                    .add(jLabel1))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(jPanel1Layout.createSequentialGroup()
                                        .add(serialSelect, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 114, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                                        .add(connectButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 110, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(textFieldBlockTo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(textFieldBlockFrom, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 79, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(chipSelect, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 433, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                    .add(jPanel1Layout.createSequentialGroup()
                                        .add(textFieldFile, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 391, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(browseButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 36, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))))
                        .add(0, 24, Short.MAX_VALUE))))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(serialSelect, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel1)
                    .add(connectButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 44, Short.MAX_VALUE)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(chipSelect, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5)
                    .add(textFieldFile, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(browseButton, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 20, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(textFieldBlockFrom, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel4)
                    .add(textFieldBlockTo, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                    .add(dumpButton, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(progressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 23, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        jLabel1.getAccessibleContext().setAccessibleName("");

        textPane.setEditable(false);
        textPane.setFont(new java.awt.Font("Monospaced", 0, 12)); // NOI18N
        jScrollPane2.setViewportView(textPane);
        log = textPane.getStyledDocument();

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jScrollPane2)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 427, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void chipSelectActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_chipSelectActionPerformed
    {//GEN-HEADEREND:event_chipSelectActionPerformed
        selectedNandConfig = (NandConfig) chipSelect.getSelectedItem();
        textFieldBlockFrom.setText(Integer.toString(0));
        textFieldBlockTo.setText(Integer.toString(selectedNandConfig.numBlocks - 1));
    }//GEN-LAST:event_chipSelectActionPerformed

    private void dumpButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_dumpButtonActionPerformed
    {//GEN-HEADEREND:event_dumpButtonActionPerformed
        if(dumpTask == null)
        {
            progressBar.setValue(0);
            dumpTask = new DumpTask(textFieldFile.getText(),
                                    Integer.parseInt(textFieldBlockFrom.getText()),
                                    Integer.parseInt(textFieldBlockTo.getText()));
            dumpTask.addPropertyChangeListener(this);
            dumpTask.execute();
        }
        else
        {
            dumpTask.abort();
        }
        updateControls();
    }//GEN-LAST:event_dumpButtonActionPerformed

    private void serialSelectActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_serialSelectActionPerformed
    {//GEN-HEADEREND:event_serialSelectActionPerformed
    }//GEN-LAST:event_serialSelectActionPerformed

    private void textFieldBlockFromFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_textFieldBlockFromFocusLost
    {//GEN-HEADEREND:event_textFieldBlockFromFocusLost
        int blockFrom = 0;
        if(textFieldBlockFrom.getText().length() > 0)
        {
            blockFrom = Integer.parseInt(textFieldBlockFrom.getText());
        }
        int blockTo = 0;
        if(textFieldBlockTo.getText().length() > 0)
        {
            blockTo = Integer.parseInt(textFieldBlockTo.getText());
        }
        if(blockFrom < 0)
        {
            blockFrom = 0;
        }
        if(blockFrom > blockTo)
        {
            blockFrom = blockTo;
        }
        textFieldBlockFrom.setText(Integer.toString(blockFrom));
    }//GEN-LAST:event_textFieldBlockFromFocusLost

    private void textFieldBlockToFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_textFieldBlockToFocusLost
    {//GEN-HEADEREND:event_textFieldBlockToFocusLost
        int blockFrom = 0;
        if(textFieldBlockFrom.getText().length() > 0)
        {
            blockFrom = Integer.parseInt(textFieldBlockFrom.getText());
        }
        int blockTo = 0;
        if(textFieldBlockTo.getText().length() > 0)
        {
            blockTo = Integer.parseInt(textFieldBlockTo.getText());
        }
        if(blockTo >= selectedNandConfig.numBlocks)
        {
            blockTo = selectedNandConfig.numBlocks - 1;
        }
        if(blockTo < blockFrom)
        {
            blockTo = blockFrom;
        }
        textFieldBlockTo.setText(Integer.toString(blockTo));
    }//GEN-LAST:event_textFieldBlockToFocusLost

    private void textFieldFileFocusLost(java.awt.event.FocusEvent evt)//GEN-FIRST:event_textFieldFileFocusLost
    {//GEN-HEADEREND:event_textFieldFileFocusLost
        // TODO add your handling code here:
    }//GEN-LAST:event_textFieldFileFocusLost

    private void saveButton1ActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_saveButton1ActionPerformed
    {//GEN-HEADEREND:event_saveButton1ActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_saveButton1ActionPerformed

    private void browseButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_browseButtonActionPerformed
    {//GEN-HEADEREND:event_browseButtonActionPerformed
        JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
        fc.setDialogTitle("Chose output file");
        if(fc.showSaveDialog(this) == JFileChooser.APPROVE_OPTION)
        {
            textFieldFile.setText(fc.getSelectedFile().getAbsolutePath());
        }
    }//GEN-LAST:event_browseButtonActionPerformed

    private void connectButtonActionPerformed(java.awt.event.ActionEvent evt)//GEN-FIRST:event_connectButtonActionPerformed
    {//GEN-HEADEREND:event_connectButtonActionPerformed
        if(serialPort != null)
        {
            serialDisconnect();
        }
        else
        {
            serialConnect();
        }
        updateControls();
    }//GEN-LAST:event_connectButtonActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton browseButton;
    private javax.swing.JComboBox chipSelect;
    private javax.swing.JButton connectButton;
    private javax.swing.JButton dumpButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JButton saveButton1;
    private javax.swing.JComboBox serialSelect;
    private javax.swing.JTextField textFieldBlockFrom;
    private javax.swing.JTextField textFieldBlockTo;
    private javax.swing.JTextField textFieldFile;
    public javax.swing.JTextPane textPane;
    // End of variables declaration//GEN-END:variables
}
